/*
 * app.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
  
#include "app.h"
#include "text.xpm"
#include "box.xpm"
#include "box2.xpm"
#include "circle.xpm"
#include "circle2.xpm"

/*
 * widgets
 */

GtkWidget *notebook = NULL;
GtkWidget *title_entry = NULL;
GtkWidget *date_entry = NULL;
GtkWidget *prio_entry = NULL;
GtkWidget *todo_check = NULL;
GtkWidget *text_entry = NULL;
GtkWidget *note_tree = NULL;      /* tree with notes on the left */
GtkWidget *b_apply = NULL;
GtkWidget *b_cancel = NULL;
GtkWidget *prio_date_box = NULL;  /* hbox used for showing/ hiding */
GtkWidget *todolist = NULL;       /* todolist on the right */
GtkWidget *status = NULL;         /* statusbar */
GtkWidget *prio_ar_down = NULL;   /* eventbox */
GtkWidget *prio_ar_up = NULL;     /* eventbox */

/*
 * pixmaps
 */

GtkWidget *pix_text_xpm;
GtkWidget *pix_box_xpm;
GtkWidget *pix_box2_xpm;
GtkWidget *pix_circle_xpm;
GtkWidget *pix_circle2_xpm;

/* ------------------------------------------------------ */
/*
 * draw the main gui
 */

GtkWidget*
yank_app_new(const gchar *geometry, const gchar *file)
{
    static GtkTargetEntry drop_types[] =
    {
        { "text/uri-list", 0, 1 },
    };
    static gint n_drop_types = sizeof (drop_types)/sizeof (drop_types [0]);
    gchar     *treetitle[] =
    {
        _("NoteTree"),
        NULL
    };
    gchar     *todotitles[] =
    {
        _("Deadline"),
        _("Priority"),
        _("Title"),
        NULL
    };
    static gint op_up = 1;         /* priority operation */
    static gint op_down = 2;       /* priority operation */
    GtkWidget *app;
    GtkWidget *hpaned1;
    GtkWidget *scrolledwindow1;
    GtkWidget *scrolledwindow2;
    GtkWidget *scrolledwindow3;
    GtkWidget *hbox;
    GtkWidget *hbox3;
    GtkWidget *vbox;
    GtkWidget *b_finish;
    GtkWidget *label;
    GdkFont   *font;
    GtkWidget *arrow_down;
    GtkWidget *arrow_up;
    
    app = gnome_app_new(PACKAGE, _(PACKAGE));
    
    gtk_window_set_policy(GTK_WINDOW(app), FALSE, TRUE, FALSE);
    gtk_window_set_default_size(GTK_WINDOW(app), 600, 400);
    gtk_window_set_wmclass(GTK_WINDOW(app), PACKAGE, PACKAGE);
    
    hpaned1 = gtk_hpaned_new();
    gtk_widget_show(hpaned1);

    /*
     * main tree
     */
    
    scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize(scrolledwindow1, 200, -1);
    gtk_widget_show(scrolledwindow1);
    gtk_container_add(GTK_CONTAINER(hpaned1), scrolledwindow1);

    note_tree = gtk_ctree_new_with_titles(1, 0, treetitle);
    gtk_widget_show(note_tree);
    gtk_container_add(GTK_CONTAINER(scrolledwindow1), note_tree);
    gtk_clist_set_column_width(GTK_CLIST(note_tree), 0, 80);
    gtk_clist_column_titles_passive(GTK_CLIST(note_tree));
    gtk_clist_set_row_height(GTK_CLIST(note_tree), 19);
    gtk_clist_set_reorderable(GTK_CLIST(note_tree), TRUE);
    gtk_clist_set_use_drag_icons(GTK_CLIST(note_tree), TRUE);
    gtk_clist_column_titles_show(GTK_CLIST(note_tree));
    gtk_ctree_set_line_style(GTK_CTREE(note_tree), GTK_CTREE_LINES_DOTTED);
    gtk_signal_connect(GTK_OBJECT(note_tree), "tree_select_row",
                       GTK_SIGNAL_FUNC (note_tree_row_selected), NULL);
    gtk_signal_connect(GTK_OBJECT(note_tree), "tree_unselect_row",
                       GTK_SIGNAL_FUNC (note_tree_row_unselected), NULL);
    /*
     * FIXME:
     * dnd on the note-tree disables reordering of notes by dnd
     */
/*     gtk_signal_connect(GTK_OBJECT(note_tree), "drag_data_received", */
/*                        GTK_SIGNAL_FUNC(cb_note_tree_drop), NULL); */
/*     gtk_drag_dest_set(note_tree, GTK_DEST_DEFAULT_MOTION | */
/*                       GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP, */
/*                       drop_types, n_drop_types, GDK_ACTION_COPY); */

    notebook = gtk_notebook_new();
    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
    gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
    gtk_widget_show(notebook);
    gtk_container_add(GTK_CONTAINER(hpaned1), notebook);

    /*
     * todo-list
     */
    
    scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow2),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(scrolledwindow2);
    gtk_container_add(GTK_CONTAINER(notebook), scrolledwindow2);
    
    todolist = gtk_clist_new_with_titles(3, todotitles);
    gtk_signal_connect(GTK_OBJECT(todolist), "select_row",
                       GTK_SIGNAL_FUNC (cb_todo_row_selected), NULL);
    gtk_signal_connect(GTK_OBJECT(todolist), "click_column",
                       GTK_SIGNAL_FUNC (cb_todo_col_selected), NULL);
    gtk_signal_connect(GTK_OBJECT(todolist), "drag_data_received",
                       GTK_SIGNAL_FUNC(cb_todo_list_drop), NULL);
    gtk_drag_dest_set(todolist, GTK_DEST_DEFAULT_MOTION |
                      GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
                      drop_types, n_drop_types, GDK_ACTION_COPY);
    gtk_widget_show(todolist);
    gtk_container_add(GTK_CONTAINER(scrolledwindow2), todolist);
    gtk_clist_column_titles_show(GTK_CLIST(todolist));
    
    /*
     * text/ data
     */
    
    vbox = gtk_vbox_new(FALSE, 3);
    gtk_widget_show(vbox);
    gtk_container_add(GTK_CONTAINER (notebook), vbox);
    
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);

    label = gtk_label_new(_("Title:"));
    font = gtk_widget_get_style(label)->font;
    gtk_widget_set_usize(label, -1, gdk_string_height(font, "W") + 10);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);

    title_entry = gtk_entry_new();
    gtk_widget_set_usize(title_entry, -1, gdk_string_height(font, "W") + 10);
    gtk_widget_show(title_entry);
    gtk_box_pack_start(GTK_BOX(hbox), title_entry, TRUE, TRUE, 5);

    todo_check = gtk_check_button_new_with_label(_("Done"));
    gtk_widget_set_usize(todo_check, -1, gdk_string_height(font, "W") + 10);
    gtk_widget_show(todo_check);
    gtk_box_pack_start(GTK_BOX(hbox), todo_check, FALSE, FALSE, 5);
    
    prio_date_box = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(prio_date_box);
    gtk_box_pack_start(GTK_BOX(vbox), prio_date_box, FALSE, FALSE, 3);

    b_finish = gtk_button_new_with_label(_("Deadline:"));
    gtk_widget_set_usize(b_finish, -1, gdk_string_height(font, "W") + 10);
    gtk_signal_connect(GTK_OBJECT(b_finish), "clicked",
                       GTK_SIGNAL_FUNC(cb_open_date_sel), app);
    gtk_widget_show(b_finish);
    gtk_box_pack_start(GTK_BOX(prio_date_box), b_finish, FALSE, FALSE, 5);

    date_entry = gtk_entry_new();
    gtk_widget_set_usize(date_entry, gdk_string_width(font, "99 WWW 9999") +
                         10, gdk_string_height(font, "W") + 10);
    gtk_widget_show(date_entry);
    gtk_box_pack_start(GTK_BOX(prio_date_box), date_entry, FALSE, FALSE, 5);
    
    label = gtk_label_new(_("Priority:"));
    gtk_widget_set_usize(label, -1, gdk_string_height(font, "W") + 10);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(prio_date_box), label, FALSE, FALSE, 5);

    prio_entry = gtk_entry_new();
    gtk_widget_set_usize(prio_entry, gdk_string_width(font, "9999") + 10,
                         gdk_string_height(font, "W") + 10);
    gtk_widget_show(prio_entry);
    gtk_box_pack_start(GTK_BOX(prio_date_box), prio_entry, FALSE, FALSE, 5);

    prio_ar_down = gtk_event_box_new();
    gtk_widget_show(prio_ar_down);
    gtk_box_pack_start(GTK_BOX(prio_date_box), prio_ar_down, FALSE, FALSE, 2);
    gtk_object_set_user_data(GTK_OBJECT(prio_ar_down), &op_down);
    gtk_signal_connect(GTK_OBJECT(prio_ar_down), "button_press_event",
                       GTK_SIGNAL_FUNC(cb_prio_adjust), prio_entry);
    
    arrow_down = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
    gtk_widget_show(arrow_down);
    gtk_container_add(GTK_CONTAINER(prio_ar_down), arrow_down);
    
    prio_ar_up = gtk_event_box_new();
    gtk_widget_show(prio_ar_up);
    gtk_box_pack_start(GTK_BOX(prio_date_box), prio_ar_up, FALSE, FALSE, 2);
    gtk_object_set_user_data(GTK_OBJECT(prio_ar_up), &op_up);
    gtk_signal_connect(GTK_OBJECT(prio_ar_up), "button_press_event",
                       GTK_SIGNAL_FUNC(cb_prio_adjust), prio_entry);
    
    arrow_up = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
    gtk_widget_show(arrow_up);
    gtk_container_add(GTK_CONTAINER(prio_ar_up), arrow_up);
    
    scrolledwindow3 = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow3),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(scrolledwindow3);
    gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow3, TRUE, TRUE, 0);
                       
    text_entry = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(text_entry), TRUE);
    gtk_widget_show(text_entry);
    gtk_container_add(GTK_CONTAINER(scrolledwindow3), text_entry);

    /* text dnd */
    gtk_signal_connect(GTK_OBJECT(text_entry), "drag_data_received",
                       GTK_SIGNAL_FUNC(cb_text_entry_drop), NULL);
    gtk_drag_dest_set(text_entry, GTK_DEST_DEFAULT_MOTION |
                      GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
                      drop_types, n_drop_types, GDK_ACTION_COPY);

    hbox3 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox3);
    gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, FALSE, 3);

    b_apply = gnome_stock_button(GNOME_STOCK_BUTTON_APPLY);
    gtk_widget_show(b_apply);
    gtk_box_pack_start(GTK_BOX(hbox3), b_apply, TRUE, FALSE, 5);
    gtk_signal_connect(GTK_OBJECT(b_apply), "clicked",
                       GTK_SIGNAL_FUNC(cb_b_apply), NULL);
        
    b_cancel = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
    gtk_widget_show(b_cancel);
    gtk_box_pack_start(GTK_BOX(hbox3), b_cancel, TRUE, FALSE, 5);
    gtk_signal_connect(GTK_OBJECT(b_cancel), "clicked",
                       GTK_SIGNAL_FUNC(cb_b_cancel), NULL);

    /*
     * a bit suboptimal
     */
    
    gtk_signal_connect(GTK_OBJECT(title_entry), "changed",
                       GTK_SIGNAL_FUNC(note_changed), NULL);
    gtk_signal_connect(GTK_OBJECT(date_entry), "changed",
                       GTK_SIGNAL_FUNC(note_changed), NULL);
    gtk_signal_connect(GTK_OBJECT(prio_entry), "changed",
                       GTK_SIGNAL_FUNC(note_changed), NULL);
    gtk_signal_connect(GTK_OBJECT(todo_check), "toggled",
                       GTK_SIGNAL_FUNC(note_changed), NULL);
    gtk_signal_connect(GTK_OBJECT(text_entry), "changed",
                       GTK_SIGNAL_FUNC(note_changed), NULL);

    /*
     * keyboard-control
     */
    
    gtk_signal_connect(GTK_OBJECT(title_entry), "activate",
                       GTK_SIGNAL_FUNC (cb_activate_apply), NULL);
    gtk_signal_connect(GTK_OBJECT(text_entry), "activate",
                       GTK_SIGNAL_FUNC (cb_activate_apply), NULL);

    /*
     * create pixmaps
     */
    
    pix_text_xpm = gnome_pixmap_new_from_xpm_d(text_xpm);
    gtk_widget_show(pix_text_xpm);
    pix_box_xpm = gnome_pixmap_new_from_xpm_d(box_xpm);
    gtk_widget_show(pix_box_xpm);
    pix_box2_xpm = gnome_pixmap_new_from_xpm_d(box2_xpm);
    gtk_widget_show(pix_box2_xpm);
    pix_circle_xpm = gnome_pixmap_new_from_xpm_d(circle_xpm);
    gtk_widget_show(pix_circle_xpm);
    pix_circle2_xpm = gnome_pixmap_new_from_xpm_d(circle2_xpm);
    gtk_widget_show(pix_circle2_xpm);

    gnome_app_set_contents(GNOME_APP(app), hpaned1);
    status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
    gnome_app_set_statusbar(GNOME_APP(app), status);
    yank_install_menus_and_toolbar(app);
    
    gtk_signal_connect(GTK_OBJECT(app), "delete_event",
                       GTK_SIGNAL_FUNC(cb_delete_event), NULL);

    gtk_clist_set_column_width(GTK_CLIST(todolist), 0,
                               gdk_string_width(font, "99 WWW 9999"));

    load_preferences();
    
    /*
     * parse geometry if given
     */
    
    if (geometry != NULL)
    {
        gint x, y, w, h;
        if (gnome_parse_geometry(geometry, &x, &y, &w, &h ))
        {
            if (x != -1)
            {
                gtk_widget_set_uposition(app, x, y);
            }
            
            if (w != -1)
            {
                gtk_window_set_default_size(GTK_WINDOW(app), w, h);
            }
        }
        else
        {
            g_error(_("Could not parse geometry string `%s'"), geometry);
        }
    }

    /*
     * load file from the command-line
     */
    
    if (file != NULL)
    {
        load_notes_from_cli((gchar *)file, GTK_CTREE(note_tree), NULL);
    }

    return (app);
}

/* ------------------------------------------------------ */
/*
 * adjust the notebook
 */

inline void
show_todo_tab(void)
{
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
}

/* ------------------------------------------------------ */
/*
 * adjust the notebook
 */

inline void
show_text_tab(void)
{
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
}

/* ------------------------------------------------------ */
/*
 * wipe the text fields
 */

void
clear_text_page(void)
{
    guint len;
    
    gtk_entry_set_text(GTK_ENTRY(title_entry), "");
    gtk_entry_set_text(GTK_ENTRY(date_entry), "");
    gtk_entry_set_text(GTK_ENTRY(prio_entry), "");
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(todo_check), FALSE);
    len = gtk_text_get_length(GTK_TEXT(text_entry));
    if (len)
    {
        gtk_text_freeze(GTK_TEXT(text_entry));
        gtk_text_set_point(GTK_TEXT(text_entry), 0);
        gtk_text_forward_delete(GTK_TEXT(text_entry), len);
        gtk_text_thaw(GTK_TEXT(text_entry));
    }
}

/* ------------------------------------------------------ */
/*
 * show the correct portion of the note
 */

void
view_note_as(_notetype type)
{
    switch (type)
    {
    case TextNote:
        gtk_widget_hide(prio_date_box);
        gtk_widget_hide(todo_check);
        break;
    case CheckNote:
        gtk_widget_hide(prio_date_box);
        gtk_widget_show(todo_check);
        break;
    case TodoNote:
        gtk_widget_show(prio_date_box);
        gtk_widget_show(todo_check);
        break;
    case None:
    default:
        g_warning(_("You shouldn't see this warning - please report."));
        break;
    };
}

/* ------------------------------------------------------ */
/*
 * insert data into a struct _note_data
 */

note_data * 
fill_note_data_from_page(_notetype notetype)
{
    note_data *note;
    
    note = (note_data *) malloc(sizeof (note_data));
    if (note == NULL)
    {
        g_warning(_("Error while allocating memory for note (0)"));
        return (NULL);
    }
    note->notetype = notetype;
    note->title = g_strdup(gtk_entry_get_text(GTK_ENTRY(title_entry)));
    if (note->title == NULL)
    {
        g_warning(_("Error while allocating memory for note (1)"));
        return (NULL);
    }
    note->date = g_strdup(gtk_entry_get_text(GTK_ENTRY(date_entry)));
    if (note->date == NULL)
    {
        g_warning(_("Error while allocating memory for note (2)"));
        return (NULL);
    }
    note->prio = g_strdup(gtk_entry_get_text(GTK_ENTRY(prio_entry)));
    if (note->prio == NULL)
    {
        g_warning(_("Error while allocating memory for note (3)"));
        return (NULL);
    }
    note->todo = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(todo_check));
    note->text = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0,
                                                 gtk_text_get_length(GTK_TEXT(
                                                     text_entry))));
    if (note->text == NULL)
    {
        g_warning(_("Error while allocating memory for note (4)"));
        return (NULL);
    }
    
    return (note);
}

/* ------------------------------------------------------ */
/*
 * display the note text
 */

void
fill_page_from_note_data(note_data *note)
{
    if (note->title != NULL)
    {
        gtk_entry_set_text(GTK_ENTRY(title_entry), note->title);
    }
    if (note->date != NULL)
    {
        gtk_entry_set_text(GTK_ENTRY(date_entry), note->date);
    }
    if (note->prio != NULL)
    {
        gtk_entry_set_text(GTK_ENTRY(prio_entry), note->prio);
    }
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(todo_check), note->todo);
    if (note->text != NULL)
    {
        gtk_text_insert(GTK_TEXT(text_entry), NULL, NULL, NULL, note->text,
                        strlen(note->text));
    }
}

/* ------------------------------------------------------ */

