/*
 * callbacks.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "callbacks.h"

/* external vars */
/* extern GtkWidget *notebook; */
extern GtkWidget *title_entry;
extern GtkWidget *date_entry;
/* extern GtkWidget *prio_entry; */
/* extern GtkWidget *todo_check; */
/* extern GtkWidget *text_entry; */
/* extern GtkWidget *prio_date_box; */
extern GtkWidget *note_tree;
extern GtkWidget *b_apply ;
extern GtkWidget *b_cancel;
extern GtkWidget *pix_text_xpm;
extern GtkWidget *pix_box_xpm;
extern GtkWidget *pix_box2_xpm;
extern GtkWidget *pix_circle_xpm;
extern GtkWidget *pix_circle2_xpm;
extern GtkWidget *todolist;
extern GtkWidget *status;

/* type of current note we're dealing with */
static _notetype notetype = None;
/* what should we do with the note? */
static _editmode editmode = NoMode;
/* current selected node in the ctree on the left */
GtkCTreeNode *selected_node = NULL;
/* last filename used for loading/ saving */
/* gchar *filename = NULL; */
/* changed-data flag */
gint   modified = FALSE;

/* ------------------------------------------------------ */
/*
 * about message
 */

void
cb_about(GtkWidget* widget, gpointer data)
{
    static GtkWidget *dialog = NULL;
    GtkWidget        *app;
    
    app = (GtkWidget*) data;
    
    if (dialog != NULL)
    {
        g_assert(GTK_WIDGET_REALIZED(dialog));
        gdk_window_show(dialog->window);
        gdk_window_raise(dialog->window);
    }
    else
    {
        const gchar *authors[] =
        {
            "Michael Humann <m.hussmann@home.ins.de>",
            NULL
        };

        dialog = gnome_about_new (_(PACKAGE), VERSION,
                                  "(C) 1999 Michael Humann", authors,
                                  ("yet another NoteKeeper.\n\
http://home.ins.de/~m.hussmann/software/yank/index.html"),
                                  "yank.png");
        
        gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                           GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
        
        gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
        
        gtk_widget_show(dialog);
    }
}

/* ------------------------------------------------------ */
/*
 * exit with check
 */

void
cb_exit(GtkWidget *widget, gpointer data)
{
    static GtkWidget *dialog = NULL;
    gint             sel;
    yank_preferences *pref;
    
    /*
     * there's an error hidden here
     */

    if (modified)
    {
        pref = get_preferences();
        if (pref->auto_save_on_exit)
        {
            cb_save_file(NULL, NULL);
        }
        
        if (dialog == NULL)
        {
            dialog = gnome_question_dialog_modal_parented(
                _("Notes modified. Do you want to save them?"), NULL, NULL,
                NULL);
            gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
            sel = gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
            if (sel == 0)
            {
                cb_save_file(NULL, NULL);
            }
            if (sel == 1)
            {
                gtk_main_quit();
            }
        }
        else
        {
            g_assert(GTK_WIDGET_REALIZED(dialog));
            gdk_window_show(dialog->window);
            gdk_window_raise(dialog->window);
            gtk_main_quit();
        }
    }
    else
    {
        gtk_main_quit();
    }
}

/* ------------------------------------------------------ */
/*
 * empty function
 */

void
nothing_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *app;
    
    app = (GtkWidget*) data;
    dialog = gnome_ok_dialog_parented(_("Not ready yet"), GTK_WINDOW(app));
}

/* ------------------------------------------------------ */
/*
 * start adding a new text-note
 */

void
cb_new_text(GtkWidget *widget, gpointer data)
{
    prepare_add(TextNote);
}

/* ------------------------------------------------------ */
/*
 * start adding a new check-note
 */

void
cb_new_check(GtkWidget *widget, gpointer data)
{
    prepare_add(CheckNote);
}

/* ------------------------------------------------------ */
/*
 * start adding a new todo-note
 */

void
cb_new_todo(GtkWidget *widget, gpointer data)
{
    prepare_add(TodoNote);
}

/* ------------------------------------------------------ */
/*
 * generic stuff when starting to add a note
 */

void
prepare_add(_notetype typ)
{
    notetype = typ;
    editmode = Add;
    clear_text_page();
    view_note_as(notetype);
    gtk_widget_set_sensitive(b_apply, FALSE);
    gtk_widget_set_sensitive(b_cancel, TRUE);
    gtk_widget_grab_focus(title_entry);
    show_text_tab();
}

/* ------------------------------------------------------ */
/*
 * apply-button in the note has been clicked
 */

void
cb_b_apply(GtkWidget* widget, gpointer data)
{
    gint ret;
    
    if (notetype == TextNote || notetype == CheckNote || notetype == TodoNote)
    {
        ret = apply_note();
        if (ret)
        {
            gnome_warning_dialog(_("Note has not been added!"));
        }        
    }
    else
    {
        g_warning(_("You shouldn't see this warning - please report."));
    }
}

/* ------------------------------------------------------ */
/*
 * cancel-button in the note has been clicked
 */

void
cb_b_cancel(GtkWidget* widget, gpointer data)
{
    editmode = NoMode;
    show_todo_tab();
}

/* ------------------------------------------------------ */
/*
 * add/ edit a note
 */

gint
apply_note(void)
{
    note_data    *note;

    note = fill_note_data_from_page(notetype);
    if (note == NULL)
    {
        return (1);
    }
    if (editmode == Add)
    {
        do_add_note(note, selected_node);
    }
    else
    {
        if (editmode == Edit)
        {
            do_edit_note(note, selected_node);
        }
    }
    show_todo_tab();
    return (0);
}

/* ------------------------------------------------------ */
/*
 * somebody as clicked into the note_tree 
 */

void
note_tree_row_selected(GtkCTree *tree, GtkCTreeNode *node, gint col,
                       gpointer gpo)
{
    note_data *note;
    
    selected_node = node;
    clear_text_page();
    note = (note_data *)gtk_ctree_node_get_row_data(GTK_CTREE(note_tree),
                                                    selected_node);
    if (note != NULL)
    {
        notetype = note->notetype;
        view_note_as(note->notetype);
        fill_page_from_note_data(note);
        gtk_widget_set_sensitive(b_apply, FALSE);
        gtk_widget_set_sensitive(b_cancel, TRUE);
        editmode = Edit;
        show_text_tab();
    }
    else
    {
        gnome_warning_dialog(_("Critical error: Note without data!"));
    }
}

/* ------------------------------------------------------ */
/*
 * somebody has unselected an item in the note_tree
 */

void
note_tree_row_unselected (GtkCTree *tree, GtkCTreeNode *node, gint col)
{
    selected_node = NULL;
    show_todo_tab();
}


/* ------------------------------------------------------ */
/*
 * the note has been changed
 */

void
note_changed(void)
{
    gtk_widget_set_sensitive(b_apply, TRUE);
}


/* ------------------------------------------------------ */
/*
 * got the delete-event from wm
 */

gint
cb_delete_event(GtkWidget* window, GdkEventAny* e, gpointer data)
{
    cb_exit(NULL, NULL);
    return (TRUE);
}

/* ------------------------------------------------------ */
/*
 * add a note to the tree
 */

GtkCTreeNode *
do_add_note(note_data *note, GtkCTreeNode *parent)
{
    gchar        *title[] = { NULL, NULL};
    GtkWidget    *icon;
    GtkCTreeNode *n_node;

    icon = choose_note_icon(note);
    title[0] = note->title;
    n_node = gtk_ctree_insert_node(GTK_CTREE(note_tree), parent,
                                   NULL, title, 2,
                                   GNOME_PIXMAP(icon)->pixmap,
                                   GNOME_PIXMAP(icon)->mask,
                                   GNOME_PIXMAP(icon)->pixmap,
                                   GNOME_PIXMAP(icon)->mask,
                                   FALSE, TRUE);
    gtk_ctree_node_set_row_data(GTK_CTREE(note_tree), n_node, note);

    /*
     * update todolist
     */
    
    if (note->notetype == TodoNote && note->todo == 0)
    {
        add_todo_item(note);
    }
    
    modified = TRUE;
    
    return (n_node);
}

/* ------------------------------------------------------ */
/*
 * edit a note
 */

void
do_edit_note(note_data *n_note, GtkCTreeNode *node)
{
    static note_data *o_note;
    GtkWidget        *icon;
    gchar            *text;
    guint8           spacing;
    GdkPixmap        *pixmap_closed;
    GdkBitmap        *mask_closed;
    GdkPixmap        *pixmap_opened;
    GdkBitmap        *mask_opened;
    gboolean         is_leaf;
    gboolean         expanded;

    o_note = (note_data *) gtk_ctree_node_get_row_data(GTK_CTREE(note_tree),
                                                       GTK_CTREE_NODE(node));
    if (o_note != NULL)
    {
        icon = choose_note_icon(n_note);
        gtk_ctree_get_node_info(GTK_CTREE(note_tree),
                                GTK_CTREE_NODE(selected_node), &text,
                                &spacing, &pixmap_closed, &mask_closed,
                                &pixmap_opened, &mask_opened, &is_leaf,
                                &expanded);
        gtk_ctree_set_node_info(GTK_CTREE(note_tree),
                                GTK_CTREE_NODE(selected_node), n_note->title,
                                2, GNOME_PIXMAP(icon)->pixmap,
                                GNOME_PIXMAP(icon)->mask,
                                GNOME_PIXMAP(icon)->pixmap,
                                GNOME_PIXMAP(icon)->mask, 0, expanded);
        gtk_ctree_node_set_row_data(GTK_CTREE(note_tree), node, n_note);
        if (n_note->notetype == TodoNote)
        {
            edit_todo_item(o_note, n_note);
        }
        modified = TRUE;
        free_note(o_note);
    }
    else
    {
        gnome_warning_dialog(_("Trying to edit NULL note!"));
    }
}

/* ------------------------------------------------------ */
/*
 * select an icon for a note
 */

GtkWidget *
choose_note_icon(note_data *note)
{
    GtkWidget *icon;
    
    switch (note->notetype)
    {
    case TextNote:
        icon = pix_text_xpm;
        break;
    case CheckNote:
        icon = note->todo ? pix_circle2_xpm : pix_circle_xpm;
        break;
    case TodoNote:
        icon = note->todo ? pix_box2_xpm : pix_box_xpm;
        break;
    default:
        icon = NULL;
        break;
    };
    return (icon);
}

/* ------------------------------------------------------ */
/*
 * show a calendar ...
 */

void
cb_open_date_sel(GtkWidget *w, gpointer p)
{
    const gchar      *buttons[] =
    {
        GNOME_STOCK_BUTTON_OK,
        GNOME_STOCK_BUTTON_CANCEL,
        NULL
    };
    static GtkWidget *dialog = NULL;
    static GtkWidget *cal = NULL;
    GtkWidget        *app;
    gchar            *txt;
    gint             year;
    gint             month;
    gint             day;
    gint             ret;
    
    /*
     * screate the dialog
     */
    
    if (dialog == NULL)
    {
        app = GTK_WIDGET(p);
        dialog = gnome_dialog_newv(_("Finish until"), buttons);
        cal = gtk_calendar_new();
        gtk_widget_show(cal);
        gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), cal, FALSE,
                           FALSE, 5);
        gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
        gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
        gnome_dialog_button_connect(GNOME_DIALOG(dialog), 0,
                                    GTK_SIGNAL_FUNC(cb_date_sel_ok), cal);
    }
    else
    {
        g_assert(GTK_WIDGET_REALIZED(dialog));
        gdk_window_show(dialog->window);
        gdk_window_raise(dialog->window);
    }
    
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    /*
     * set the date
     */
    
    txt = gtk_entry_get_text(GTK_ENTRY(date_entry));
    if (strlen(txt))
    {
        ret = date2num(txt, &day, &month, &year);
    }
    else
    {
        ret = date2num(current_date(), &day, &month, &year);
    }
    if (ret == 0)
    {
        gtk_calendar_select_month(GTK_CALENDAR(cal), month, year);
        gtk_calendar_select_day(GTK_CALENDAR(cal), day);
    }
    
    gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

/* ------------------------------------------------------ */
/*
 * ok pressed in the calendar
 */

void
cb_date_sel_ok(GtkWidget *w, gpointer p)
{
    gint year;
    gint month;
    gint day;
    
    gtk_calendar_get_date(GTK_CALENDAR(p), &year, &month, &day);
    gtk_entry_set_text(GTK_ENTRY(date_entry), num2date(day, month, year));
}

/* ------------------------------------------------------ */
/*
 * remove note from mem
 */

void
free_note(note_data *data)
{
    if (data)
    {
        if (data->title)
        {
            free(data->title);
        }
        if (data->prio)
        {
            free(data->prio);
        }
        if (data->date)
        {
            free(data->date);
        }
        if (data->text)
        {
            free(data->text);
        }
        free(data);
    }
}

/* ------------------------------------------------------ */
/*
 * clicks in the todo-list
 */

void
cb_todo_row_selected(GtkCList *list, gint row, gint col, GdkEvent *evt)
{
    note_data    *note;
    GtkCTreeNode *tnode;
    
    if (evt && evt->type == GDK_2BUTTON_PRESS)
    {
        clear_text_page();
        note = (note_data *)gtk_clist_get_row_data(GTK_CLIST(list), row);
        tnode = gtk_ctree_find_by_row_data(GTK_CTREE(note_tree), NULL, note);
        if (tnode != NULL)
        {
            gtk_ctree_select(GTK_CTREE(note_tree), tnode);
        }
    }
}

/* ------------------------------------------------------ */
/*
 * a todo-list column-title has been clicked
 */

void
cb_todo_col_selected(GtkCList *list, gint col)
{
    switch (col)
    {
    case 0:
        cb_sort_todo_date(NULL, NULL);
        break;
    case 1:
        cb_sort_todo_prio(NULL, NULL);
        break;
    case 2:                       /* ok, this is text ... */
        cb_sort_todo_date_prio(NULL, NULL);
        break;
    };
}

/* ------------------------------------------------------ */
/*
 * sort the todolist by date
 */

void
cb_sort_todo_date(GtkWidget *w, gpointer p)
{
    gtk_clist_freeze(GTK_CLIST(todolist));
    gtk_clist_set_compare_func(GTK_CLIST(todolist),
                               (GtkCListCompareFunc)(*sort_todo_items_date));
    gtk_clist_sort(GTK_CLIST(todolist));
/*     gtk_clist_set_auto_sort(GTK_CLIST(todolist), TRUE);  */
    gtk_clist_thaw(GTK_CLIST(todolist));
}

/* ------------------------------------------------------ */
/*
 * sort the todolist by priority
 */

void
cb_sort_todo_prio(GtkWidget *w, gpointer p)
{
    gtk_clist_freeze(GTK_CLIST(todolist));
    gtk_clist_set_compare_func(GTK_CLIST(todolist),
                               (GtkCListCompareFunc)(*sort_todo_items_prio));
    gtk_clist_sort(GTK_CLIST(todolist));
/*     gtk_clist_set_auto_sort(GTK_CLIST(todolist), TRUE);  */
    gtk_clist_thaw(GTK_CLIST(todolist));
}

/* ------------------------------------------------------ */
/*
 * sort the todolist by date& priority
 */

void
cb_sort_todo_date_prio(GtkWidget *w, gpointer p)
{
    gtk_clist_freeze(GTK_CLIST(todolist));
    gtk_clist_set_compare_func(GTK_CLIST(todolist),
                               (GtkCListCompareFunc)
                               (*sort_todo_items_date_prio));
    gtk_clist_sort(GTK_CLIST(todolist));
/*     gtk_clist_set_auto_sort(GTK_CLIST(todolist), TRUE);  */
    gtk_clist_thaw(GTK_CLIST(todolist));
}

/* ------------------------------------------------------ */
/*
 * print to current note
 */

void
cb_print_note(GtkWidget *w, gpointer p)
{
    static gchar *done[] = { "no", "yes" };
    note_data    *data;
    FILE         *str;
    gchar        *txt;
    
    data = gtk_ctree_node_get_row_data(GTK_CTREE(note_tree),
                                       GTK_CTREE_NODE(selected_node));
    if (data)
    {
        /*
         * FIXME: select printer in properties
         */
        
        str = popen("lpr", "w");
        if (str)
        {
            fprintf(str, "Title: %s\n", data->title);
            if (data->notetype == CheckNote)
            {
                fprintf(str, "Done: %s\n", done[data->todo]);
            }
            else
            {
                if (data->notetype == TodoNote)
                {
                    fprintf(str, "Finish until: %s\t Priority: %s\tDone: %s\n",
                            data->date, data->prio, done[data->todo]);
                    
                }
            }
            fprintf(str, "%s\n", data->text);
            pclose(str);

            txt = g_strdup_printf("Printed: %s", data->title);
            gnome_appbar_set_status(GNOME_APPBAR(status), txt);
            g_free(txt);
        }
        else
        {
            gnome_warning_dialog_parented(_("Print: Can't popen() lpr!"),
                                          GTK_WINDOW(p));
        }
    }
    else
    {
        gnome_ok_dialog_parented(_("Print: Select a note first!"),
                                 GTK_WINDOW(p));
    }
}

/* ------------------------------------------------------ */
/*
 * proxy func
 */

void
cb_modify_text(GtkWidget *w, gpointer p)
{
    modify_note(TextNote, GTK_WINDOW(p));
}

/* ------------------------------------------------------ */
/*
 * proxy func
 */

void
cb_modify_check(GtkWidget *w, gpointer p)
{
    modify_note(CheckNote, GTK_WINDOW(p));
}

/* ------------------------------------------------------ */
/*
 * proxy func
 */

void
cb_modify_todo(GtkWidget *w, gpointer p)
{
    modify_note(TodoNote, GTK_WINDOW(p));
}

/* ------------------------------------------------------ */
/*
 * change a notes type
 */

void
modify_note(_notetype t_type, GtkWindow *w)
{
    note_data *data;
    GtkWidget *icon;
    gchar     *text;
    guint8    spacing;
    GdkPixmap *pixmap_closed;
    GdkBitmap *mask_closed;
    GdkPixmap *pixmap_opened;
    GdkBitmap *mask_opened;
    gboolean  is_leaf;
    gboolean  expanded;
    
    data = (note_data *) gtk_ctree_node_get_row_data(
        GTK_CTREE(note_tree), GTK_CTREE_NODE(selected_node));

    if (data)
    {
        /*
         * update the todo-list
         */
        
        if (t_type == TodoNote && data->todo == 0)
        {
            add_todo_item(data);
        }
        if (data->notetype == TodoNote && data->todo == 0)
        {
            delete_todo_item(data);
        }
        
        data->notetype = t_type;
        modified = TRUE;

        /*
         * update the tree
         */
        
        icon = choose_note_icon(data);
        gtk_ctree_get_node_info(GTK_CTREE(note_tree),
                                GTK_CTREE_NODE(selected_node), &text,
                                &spacing, &pixmap_closed, &mask_closed,
                                &pixmap_opened, &mask_opened, &is_leaf,
                                &expanded);
        gtk_ctree_set_node_info(GTK_CTREE(note_tree),
                                GTK_CTREE_NODE(selected_node), data->title,
                                2, GNOME_PIXMAP(icon)->pixmap,
                                GNOME_PIXMAP(icon)->mask,
                                GNOME_PIXMAP(icon)->pixmap,
                                GNOME_PIXMAP(icon)->mask, 0, expanded);
        gtk_ctree_select(GTK_CTREE(note_tree), GTK_CTREE_NODE(selected_node));
    }
    else
    {
        gnome_warning_dialog_parented(_("Trying to modify NULL note!"), w);
    }
}

/* ------------------------------------------------------ */
/*
 * drop-request on the text-entry
 */

void
cb_text_entry_drop(GtkWidget *entry, GdkDragContext *context, gint x, gint y,
                   GtkSelectionData *selection_data, guint info, guint time)
{
    GList *names;

    names = gnome_uri_list_extract_filenames((char *)selection_data->data);
    while (names)
    {
        txt_append_file(GTK_TEXT(entry), (gchar *)names->data);
        names = names->next;
    }
    note_changed();
}

/* ------------------------------------------------------ */
/*
 * drop-request on the todo-list
 */

void
cb_todo_list_drop(GtkWidget *todo, GdkDragContext *context, gint x, gint y,
                  GtkSelectionData *selection_data, guint info, guint time)
{
    GList     *names;
    note_data *note;
    
    names = gnome_uri_list_extract_filenames((char *)selection_data->data);
    while (names)
    {
        note = make_note_from_file((gchar *)names->data);
        do_add_note(note, selected_node);
        names = names->next;
    }
    modified = TRUE;
}

/* ------------------------------------------------------ */
/*
 * drop-request on the note-tree
 */

void
cb_note_tree_drop(GtkWidget *tree, GdkDragContext *context, gint x, gint y,
                  GtkSelectionData *selection_data, guint info, guint time)
{
    GList *names;
    
    names = gnome_uri_list_extract_filenames((char *)selection_data->data);
    while (names)
    {
        load_notes_from_cli((gchar *)names->data, GTK_CTREE(tree),
                            selected_node);
        names = names->next;
    }
    modified = TRUE;
}

/* ------------------------------------------------------ */
/*
 * [in|de]crement the priority
 */

void
cb_prio_adjust(GtkWidget *widget, GdkEventButton *evt, gpointer entry)
{
    gint  prio;
    gchar *txt;
    gchar *out;
    gint  *op;
    
    op = (gint *) gtk_object_get_user_data(GTK_OBJECT(widget));
    txt = gtk_entry_get_text(GTK_ENTRY(entry));
    prio = strtol(txt, NULL, 10);
    switch (*op)
    {
    case 1:
        prio++;
        break;
    case 2:
        prio--;
        break;
    default:
        g_warning(_("cb_prio_adjust: strange things seem to happen"));
    };
    out = g_strdup_printf("%d", prio);
    gtk_entry_set_text(GTK_ENTRY(entry), out);
    g_free(out);
}

/* ------------------------------------------------------ */
/*
 * activate-signal from title_entry, text_entry
 */

void
cb_activate_apply(GtkWidget *w, gpointer p)
{
    if (GTK_WIDGET_IS_SENSITIVE(b_apply))
    {
        gtk_widget_grab_focus(b_apply);
    }
    else
    {
        gtk_widget_grab_focus(b_cancel);
    }
}

/* ------------------------------------------------------ */
