/*
 * daycalc.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "daycalc.h"

static char *monthlist[] =
{
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec"
};

/* -------------------------------------------------------- */

int
daynr(int day, int month, int year)
{
    int d, e;

    d = (month + 10) / 13;
    e = day + (611 * (month + 2)) / 20 - 2 * d - 91;
    return  (e + leapyear(year) * d);
}

/* -------------------------------------------------------- */

int
leapyear(int year)
{
    if ( (( year % 4 ==0) && (year % 100 != 0)) || (year % 400 == 0) )
    {
        return (1);
    }
    return (0);
}

/* -------------------------------------------------------- */

int
month_in_year(int year, int n)
{
    int a;

    a = leapyear(year);
    if ( n > 59+a )
    {
	n += 2-a;
    }
    n += 91;
    return ((20*n)/611 - 2);
}

/* -------------------------------------------------------- */

int
day_in_year(int year, int n)
{
    int a, m;
    a = leapyear(year);
    if ( n > 59+a )
    {
	n += 2-a;
    }
    n += 91;
    m = (20*n)/611;
    return n - (611*m)/20;
}

/* -------------------------------------------------------- */
/* get the number of days since 1.1.1970 and return 
   a string 'dd month yyyy' */

char *
days2date(long int d_count)
{
    int         days;
    int         year = 1970;

    d_count++;
    while (d_count > (days = leapyear(year) ? 366 : 365))
    {
	d_count -= days;
	year++;
    }
    return (num2date(day_in_year(year, d_count),
                     (month_in_year(year, d_count) -1) ,year));
}


/* -------------------------------------------------------- */
/* get a string 'dd month yyyy' and return 
   the number of days since 1.1.1970  */

long int
date2days(char *date)
{
    int      t_year = 1970;
    int      d_days = 0;
    int      c_days = 0;
    long int ret;
    int      succ;
    int      day;
    int      month;
    int      year;
    char     *p;

    p = date;
    ret = 0;
    succ = date2num(p, &day, &month, &year);
    if (!succ)
    {
        long int day_nr;
        while ((t_year != year) && (d_days = leapyear(t_year) ? 366 : 365))
        {
            c_days = c_days + d_days;
            t_year++;
        }
        day_nr = daynr(day, (month+1), year);
        day_nr = day_nr + c_days;
        ret = day_nr - 1;
    }
    return (ret);
}

/* -------------------------------------------------------- */
/* return a nice formatted string with the current date
   like '31 Jan 2001 */

char *
current_date(void)
{
    struct tm *stm;
    time_t    xtt;
    
    xtt = time(&xtt);
    stm = localtime((const time_t *) &xtt);

    return (num2date(stm->tm_mday, stm->tm_mon, (1900+stm->tm_year)));
}

/* -------------------------------------------------------- */
/* converts *str into integers - returns 0 upon success */

int
date2num(char *str, int *day, int *month, int *year)
{
    char *p;
    int  ret;
    int  found;
    static char errmsg1 [] = "date2num: day not found\n";
    static char errmsg2 [] = "date2num: month not found\n";
    
    p = str;
    ret = 0;
    found = 0;
    
    if (strlen(p))
    {
	char *p2 = p;
	/* skip leading zeroes */
	while (*p == '0')
        {
	    p++;
	}
	/* search 1.st space */
	while ((*p != '\0') && (*p != ' '))
        {
            p++;
	}
	/* Check if we've already run off the end of the string */
	if (*p == '\0')
        {
            write(STDERR_FILENO, errmsg1, strlen(errmsg1));
            return (-1);
	}
        else
        {
            *p = '\0';
	}
	*day = atoi(p2);
	*p = ' ';
	p++;   /* skip space */
	p2 = p;
	/* search 2.nd space */
	while ((*p != '\0') && (*p != ' '))
        {
	    p++;
	}
	/* Check if we've already run off the end of the string */
	if (*p == '\0')
        {
            write(STDERR_FILENO, errmsg1, strlen(errmsg1));
            return (-1);
	}
        else
        {
            *p = '\0';
        }
        *month = 0;
	/* search month */
	while ((!found) && (*month < 12))
        {
	    if (!strcmp(p2, monthlist[*month]))
            {
		found = 1;
	    }
            else
            {
		(*month)++;
	    }
	}
	if (!found)
        {
            write(STDERR_FILENO, errmsg2, strlen(errmsg2));
	    ret = -1;
	}
        else
        {
	    *p = ' ';
	    p++;                  /* skip space */
	    p2 = p;      
	    *year = atoi(p2);     /* get year */
            if (*year > 32767)
            {
                *year = 32767;
            }
	}
    }
    
    return (ret);
}

/* -------------------------------------------------------- */
/* converts integer to a string *wow* */

char*
num2date(int day, int month, int year)
{
    static char t_buffer[20];
    
    snprintf(t_buffer, (19), "%d %s %d", day, monthlist[month], year);
    t_buffer[19] = '\0';
    
    return (t_buffer);
}

/* ------------------------------------------------------ */
