/*
 * deleteop.h
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "deleteop.h"

extern GtkWidget    *note_tree;
extern GtkWidget    *status;
/* current selected node in the ctree on the left */
extern GtkCTreeNode *selected_node;
/* changed-data flag */
extern gint         modified;

/* ------------------------------------------------------ */
/*
 * show the delete-dialog
 */

void
cb_open_delete_sel(GtkWidget *w, gpointer p)
{
    const gchar      *buttons[] =
    {
        GNOME_STOCK_BUTTON_OK,
        GNOME_STOCK_BUTTON_CANCEL,
        NULL
    };
    static GtkWidget *dialog = NULL;
    GtkWidget        *app;
    GtkWidget        *frame;
    GtkWidget        *vbox;
    GSList           *rmop;
    GtkWidget        *rmone;
    GtkWidget        *rmtree;
    
    app = GTK_WIDGET(p);
    
    if (selected_node == NULL)
    {
        gnome_ok_dialog_parented(_("Select a note first!"), GTK_WINDOW(app));
        return;
    }
    
    if (dialog == NULL)
    {
        dialog = gnome_dialog_newv(_("Delete note(s)"), buttons);
        frame = gtk_frame_new(_("Delete"));
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), frame, FALSE,
                           FALSE, 5);
        vbox = gtk_vbox_new(FALSE, 0);
        gtk_widget_show(vbox);
        gtk_container_add(GTK_CONTAINER(frame), vbox);
        rmone = gtk_radio_button_new_with_label(NULL, _("this note"));
        gtk_widget_show(rmone);
        gtk_box_pack_start(GTK_BOX(vbox), rmone, FALSE, FALSE, 5);
        rmop = gtk_radio_button_group(GTK_RADIO_BUTTON(rmone));
        rmtree = gtk_radio_button_new_with_label(rmop, _("this subtree"));
        gtk_widget_show(rmtree);
        gtk_box_pack_start(GTK_BOX(vbox), rmtree, FALSE, FALSE, 5);
        rmop = gtk_radio_button_group(GTK_RADIO_BUTTON(rmtree));
        
        gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
        gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
        gnome_dialog_button_connect(GNOME_DIALOG(dialog), 0,
                                    GTK_SIGNAL_FUNC(cb_delete_sel_ok), rmone);
    }
    else
    {
        g_assert(GTK_WIDGET_REALIZED(dialog));
        gdk_window_show(dialog->window);
        gdk_window_raise(dialog->window);
    }

    gnome_dialog_set_default(GNOME_DIALOG(dialog), 1);
    gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

/* ------------------------------------------------------ */
/*
 * users wants to delete
 */

void
cb_delete_sel_ok(GtkWidget *w, gpointer p)
{
    GtkCTreeNode *childs;
    GtkCTreeNode *parent;
    
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(p)))
    {
        /*
         * delete 1 note
         */
        
        parent = GTK_CTREE_ROW(selected_node)->parent;

        /* prevents gtk from warning us */
        if (GTK_CTREE_ROW(selected_node)->children)
        {
            /* move childs */
            while ((childs = GTK_CTREE_ROW(selected_node)->children) != NULL)
            {
                gtk_ctree_move(GTK_CTREE(note_tree), childs, parent, NULL);
            }
        }

        free_tree_node(GTK_CTREE(note_tree), selected_node, NULL);
        
    }
    else
    {
        /*
         * delete the subtree
         */
        
        clear_tree_from_node(GTK_CTREE(note_tree),
                             GTK_CTREE_NODE(selected_node));
    }
    
    modified = TRUE;
    show_todo_tab();
}

/* ------------------------------------------------------ */
/*
 * delete the whole tree
 */

void
cb_clear_tree(GtkWidget *w, gpointer p)
{
    GtkWidget *dialog;
    gint      sel;
    GtkWidget *app;
    
    app = GTK_WIDGET(p);
    if (GTK_CLIST(note_tree)->rows > 0)
    {
        dialog = gnome_question_dialog_modal_parented(
            _("Delete all note(s)?"), NULL, NULL,
            GTK_WINDOW(app));
        gnome_dialog_set_default(GNOME_DIALOG(dialog), 1);
        sel = gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
        if (sel == 0)
        {
            clear_tree_from_node(GTK_CTREE(note_tree), NULL);
            show_todo_tab();
            gnome_appbar_set_status(GNOME_APPBAR(status), _("*Wow* Rambo."));
            modified = TRUE;
        }
    }
    else
    {
        gnome_ok_dialog_parented(_("Nothing to delete!"), GTK_WINDOW(app));
    }
}

/* ------------------------------------------------------ */
/*
 * show 'expire note(s)'
 */

void
cb_delete_finished_notes(GtkWidget *w, gpointer p)
{
    GtkWidget *dialog;
    GtkWidget *rmroot;
    GtkWidget *rmsel;
    GSList    *rmop;
    GtkWidget *frame;
    GtkWidget *vbox;
    
    if (GTK_CLIST(note_tree)->rows < 1)
    {
        gnome_ok_dialog_parented(_("No notes to expire."), GTK_WINDOW(p));
    }
    else
    {
        frame = gtk_frame_new(_("Start expiring notes from"));
        gtk_widget_show(frame);
        vbox = gtk_vbox_new(FALSE, 0);
        gtk_widget_show(vbox);
        gtk_container_add(GTK_CONTAINER(frame), vbox);
        rmroot = gtk_radio_button_new_with_label(NULL, _("the root of the \
tree"));
        gtk_widget_show(rmroot);
        gtk_box_pack_start(GTK_BOX(vbox), rmroot, FALSE, FALSE, 5);
        rmop = gtk_radio_button_group(GTK_RADIO_BUTTON(rmroot));
        rmsel = gtk_radio_button_new_with_label(rmop, _("current selection"));
        gtk_widget_show(rmsel);
        gtk_box_pack_start(GTK_BOX(vbox), rmsel, FALSE, FALSE, 5);
        rmop = gtk_radio_button_group(GTK_RADIO_BUTTON(rmsel));
        dialog = gnome_ok_cancel_dialog_parented(
            NULL, (GnomeReplyCallback)(cb_delete_fin_ok), rmroot,
            GTK_WINDOW(p));
        gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), frame, FALSE,
                           FALSE, 5);
        
        gtk_widget_grab_focus(rmroot);
        gtk_widget_show(dialog);
    }
}

/* ------------------------------------------------------ */
/*
 * expire notes
 */

void
cb_delete_fin_ok(gint  reply, gpointer p)
{
    if (reply == 0)
    {
        gtk_clist_freeze(GTK_CLIST(note_tree));
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(p)))
        {
            gtk_ctree_post_recursive(GTK_CTREE(note_tree), NULL,
                                     (*expire_tree_node), NULL);
        }
        else
        {
            gtk_ctree_post_recursive(GTK_CTREE(note_tree),
                                     GTK_CTREE_NODE(selected_node),
                                     (*expire_tree_node), NULL);
        }
        gtk_clist_thaw(GTK_CLIST(note_tree));
    }
}

/* ------------------------------------------------------ */
/*
 * delete a note& data
 */

void
free_tree_node(GtkCTree *tree, GtkCTreeNode *node, gpointer p)
{
    static note_data *data;

    data = gtk_ctree_node_get_row_data(GTK_CTREE(tree), GTK_CTREE_NODE(node));

    /*
     * update todolist
     */

    if (data->notetype == TodoNote && data->todo == 0)
    {
        delete_todo_item(data);
    }
    
    gtk_ctree_remove_node(tree, node);
    free_note(data);
}

/* ------------------------------------------------------ */
/*
 * expire note(s)
 */

void
expire_tree_node(GtkCTree *tree, GtkCTreeNode *node, gpointer p)
{
    GtkCTreeNode *childs;
    GtkCTreeNode *parent;
    note_data    *data;

    data = gtk_ctree_node_get_row_data(GTK_CTREE(tree), GTK_CTREE_NODE(node));

    if (data->todo == 1 &&
        (data->notetype == TodoNote || data->notetype == CheckNote))
    {
        
        /* prevents gtk from warning us */
        if (GTK_CTREE_ROW(node)->children)
        {
            parent = GTK_CTREE_ROW(node)->parent;
            /* move childs */
            while ((childs = GTK_CTREE_ROW(node)->children) != NULL)
            {
                gtk_ctree_move(GTK_CTREE(tree), childs, parent, NULL);
            }
        }
        free_tree_node(GTK_CTREE(tree), node, NULL);
    }
}

/* ------------------------------------------------------ */
/*
 * clear a subtree
 */

void
clear_tree_from_node(GtkCTree *tree, GtkCTreeNode *node)
{
    gtk_clist_freeze(GTK_CLIST(tree));
    gtk_ctree_post_recursive(tree, node, (*free_tree_node), NULL);
    gtk_clist_thaw(GTK_CLIST(tree));
}

/* ------------------------------------------------------ */

