/*
 * preferences.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "preferences.h"

static yank_preferences preferences;
static GnomePropertyBox *pref_dialog;
static GtkWidget        *b_auto_save_on_exit;
static GtkWidget        *b_auto_save_preferences;

/* ------------------------------------------------------ */
/*
 * select preferences for yank
 */

void
cb_pref_dialog(GtkWidget *w, gpointer p)
{
    GtkWidget *title;
    
    pref_dialog = GNOME_PROPERTY_BOX(gnome_property_box_new());
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "apply",
                       GTK_SIGNAL_FUNC(cb_pref_apply), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "destroy",
                       GTK_SIGNAL_FUNC(cb_pref_cancel), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "delete_event",
                       GTK_SIGNAL_FUNC(gtk_false), NULL);

    title = gtk_label_new(_("Misc"));
    gtk_notebook_append_page(GTK_NOTEBOOK((pref_dialog)->notebook),
                             pref_misc_p(), title);

    gtk_signal_connect(GTK_OBJECT(b_auto_save_on_exit), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(b_auto_save_preferences), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);

    gtk_widget_show_all(GTK_WIDGET(pref_dialog));
}

/* ------------------------------------------------------ */
/*
 * apply preferences
 */

void
cb_pref_apply(GnomePropertyBox *pref, gint page, gpointer data)
{
    preferences.auto_save_on_exit = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_auto_save_on_exit));
    preferences.auto_save_preferences = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_auto_save_preferences));
    
    if (preferences.auto_save_preferences == TRUE)
    {
        save_preferences(get_preferences());
    }
}

/* ------------------------------------------------------ */
/*
 * cancel preferences
 */

void
cb_pref_cancel(GtkWidget *w, gpointer p)
{
    gtk_widget_destroy(GTK_WIDGET(p));
    p = NULL;
}

/* ------------------------------------------------------ */
/*
 * mixed preferences tab
 */

GtkWidget *
pref_misc_p(void)
{
    GtkWidget *vbox;
    GtkWidget *vbox1;
    GtkWidget *frame;
    
    vbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox);
    
    frame = gtk_frame_new(_("Saving"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
    
    vbox1 = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);
    
    b_auto_save_on_exit = gtk_check_button_new_with_label(
        _("Autosave notes on exit"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_auto_save_on_exit),
                                 preferences.auto_save_on_exit);
    
    gtk_widget_show(b_auto_save_on_exit);
    gtk_box_pack_start(GTK_BOX(vbox1), b_auto_save_on_exit, FALSE, TRUE, 2);
    
    b_auto_save_preferences = gtk_check_button_new_with_label(
        _("Autosave preferences"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_auto_save_preferences),
                                 preferences.auto_save_preferences);
    gtk_widget_show(b_auto_save_preferences);
    gtk_box_pack_start(GTK_BOX(vbox1), b_auto_save_preferences,
                       FALSE, TRUE, 2);
    
    return (vbox);
}

/* ------------------------------------------------------ */
/*
 * return pointer to preferences
 */

inline
yank_preferences*
get_preferences(void)
{
    return (&preferences);
}

/* ------------------------------------------------------ */
/*
 * load yanks preferences
 */

void
load_preferences(void)
{
    gnome_config_push_prefix("/yank/Global/");
    
    preferences.auto_save_on_exit = gnome_config_get_int("auto save on exit");
    preferences.auto_save_preferences = gnome_config_get_int
        ("auto save preferences");

    gnome_config_pop_prefix();
    gnome_config_sync();
}

/* ------------------------------------------------------ */
/*
 * save yanks preferences
 */

void
save_preferences(yank_preferences *prefs)
{
    gnome_config_push_prefix("/yank/Global/");
    
    gnome_config_set_int("auto save on exit", prefs->auto_save_on_exit);
    gnome_config_set_int("auto save preferences",
                         prefs->auto_save_preferences);
    
    gnome_config_pop_prefix();
    gnome_config_sync();
}

/* ------------------------------------------------------ */
/*
 * save from the menu
 */

void
cb_save_preferences(GtkWidget *w, gpointer p)
{
    save_preferences(get_preferences());
}

/* ------------------------------------------------------ */
/*
 * preferences have been changed 
 */

void
pref_changed(GtkWidget *w, GnomePropertyBox *prefs)
{
  gnome_property_box_changed(prefs);
}

/* ------------------------------------------------------ */

